#!/usr/bin/env python
"""Created by Thomas Rostrup Andersen on 11/11/2016.
Copyright (C) 2016 Thomas Rostrup Andersen. All rights reserved."""

import time
import roslib 
import rospy
import actionlib
from std_msgs.msg import String
from cyborg_controller.msg import StateMachineAction, StateMachineGoal, StateMachineResult, StateMachineFeedback, EmotionalState, EmotionalFeedback, SystemState

__author__ = "Thomas Rostrup Andersen"
__copyright__ = "Copyright (C) 2016 Thomas Rostrup Andersen"
#__license__ = ""
__version__ = "0.0.2"
__all__ = []

class ConversationServer():
	"""ConversationServer"""
	
	current_emotion = "neutral"

	aborted = False # Set true if state needs to abort, see state loop.
	timeout = 120 # (secunds)

	# The initzialzation. Sets up all the ROS topics.
	def __init__(self):
		self.server_conversation = actionlib.SimpleActionServer(rospy.get_name() + "/conversation", StateMachineAction, execute_cb=self.server_conversation_callback, auto_start = False)
		self.server_conversation.start()
		self.emotion_publisher = rospy.Publisher("/cyborg_controller/emotional_feedback", EmotionalFeedback, queue_size=100)
		self.event_publisher = rospy.Publisher("/cyborg_controller/register_event", String, queue_size=100)
		self.speech_publisher = rospy.Publisher("/cyborg_text_to_speech/text_to_speech", String, queue_size=100)
		self.emotion_subscriber = rospy.Subscriber("/cyborg_controller/emotional_state", EmotionalState, self.emotion_callback, queue_size=100)
		self.text_subscriber = rospy.Subscriber("/text_from_speech", String, self.text_callback, queue_size=100)
		rospy.loginfo("ConversationServer: Activated")


	# Send the change (delta) in the emotional values to the controller (motion system)
	def send_emotion(self, pleasure, arousal, dominance):
		msg = EmotionalFeedback()
		msg.delta_pleasure = pleasure
		msg.delta_arousal = arousal
		msg.delta_dominance = dominance 
		self.emotion_publisher.publish(msg)


	# Called when the controller (state machine) sets the conversation state as active
	def server_conversation_callback(self, goal):
		rospy.loginfo("ConversationServer: Executing conversation state...")

		# This is the state loop
		rate = rospy.Rate(.5) # (hz)
		start = time.time() # Prevent eternal looping
		while not rospy.is_shutdown():

			# Do some conversational stuff here...

			# Checkin with the controller
			if self.server_conversation.is_preempt_requested():
				self.server_conversation.set_preempted()
				return
			if self.aborted: # This is currently always False (should be set True if the conversation state needs to abort)
				self.aborted = False
				self.server_conversation.set_aborted()
				return
			end = time.time()
			if (end - start > self.timeout):
				self.server_conversation.set_aborted()
				return
			rate.sleep()


	# Updates the current emotion when the emotion subscriber recives data from the controller (emotion system)
	def emotion_callback(self, data):
		self.current_emotion = data.to_emotional_state


	# Called when the speech to text publishes new text
	def text_callback(self, data):
		self.text = data.data
		rospy.logdebug("ConversationServer: Recived text from stt: " + str(self.text))
		self.event_publisher.publish("conversation_interest")

	




