#!/usr/bin/env python
"""Created by Thomas Rostrup Andersen on 11/11/2016.
Copyright (C) 2016 Thomas Rostrup Andersen. All rights reserved."""

import time
import roslib 
import rospy
import actionlib
from std_msgs.msg import String
from cyborg_controller.msg import StateMachineAction, StateMachineGoal, StateMachineResult, StateMachineFeedback, EmotionalState, EmotionalFeedback, SystemState

__author__ = "Thomas Rostrup Andersen"
__copyright__ = "Copyright (C) 2016 Thomas Rostrup Andersen"
#__license__ = ""
__version__ = "0.0.2"
__all__ = []

class SimonSaysServer():
	"""SimonSaysServer"""
	
	current_emotion = "neutral"

	aborted = False # Set true if state needs to abort, see state loop.
	timeout = 20 # (secunds)

	# The initzialzation. Sets up all the ROS topics.
	def __init__(self):
		self.server_simon_says = actionlib.SimpleActionServer(rospy.get_name() + "/simon_says", StateMachineAction, execute_cb=self.server_simon_says_callback, auto_start = False)
		self.server_simon_says.start()
		self.emotion_publisher = rospy.Publisher("/cyborg_controller/emotional_feedback", EmotionalFeedback, queue_size=100)
		self.event_publisher = rospy.Publisher('/cyborg_controller/register_event', String, queue_size=100)
		self.emotion_subscriber = rospy.Subscriber("/cyborg_controller/emotional_state", EmotionalState, self.emotion_callback, queue_size=100)
		self.speech_publisher = rospy.Publisher("/cyborg_text_to_speech/text_to_speech", String, queue_size=100)
		rospy.loginfo("SimonSaysServer: Activated")


	# Send the change (delta) in the emotional values to the controller (emotion system)
	def send_emotion(self, pleasure, arousal, dominance):
		msg = EmotionalFeedback()
		msg.delta_pleasure = pleasure
		msg.delta_arousal = arousal
		msg.delta_dominance = dominance 
		self.emotion_publisher.publish(msg)


	# Called when the controller (state machine) sets the simon_says state as active
	def server_simon_says_callback(self, goal):
		rospy.loginfo("SimonSaysServer: Executing simon_says state...")

		self.speech_publisher.publish("Do you want to play simon says?")
		time.sleep(5.0)
		self.server_simon_says.set_aborted()

		# # This is the state loop
		# rate = rospy.Rate(0.5) # (hz)
		# start = time.time() # Prevent eternal looping
		# while not rospy.is_shutdown():

		# 	# Do some simon_says stuff here...
			

		# 	# Checkin with the controller
		# 	if self.server_simon_says.is_preempt_requested():
		# 		self.server_simon_says.set_preempted()
		# 		return
		# 	if self.aborted: # This is currently always False (should be set True if the simon_says state needs to abort)
		# 		self.aborted = False
		# 		self.server_simon_says.set_aborted()
		# 		return
		# 	if (time.time()- start > self.timeout):
		# 		self.server_simon_says.set_aborted()
		# 		return
		# 	rate.sleep()
		

	# Updates the current emotion when the emotion subscriber recives data from the controller (emotion system)
	def emotion_callback(self, data):
		self.current_emotion = data.to_emotional_state

	




